#include <U8g2lib.h>
#include <Wire.h>
#include <Keyboard.h>
#include <Keypad.h>

const byte ROWS = 6; //four rows
const byte COLS = 4; //three columns
char keys[ROWS][COLS] = {
  {1,7,13,19},
  {2,8,14,20},
  {3,9,15,21},
  {4,10,16,22},
  {5,11,17,23},
  {6,12,18,24}
};
byte rowPins[ROWS] = {7, 6, 5, 4, 3, 2}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {17,16,15,14}; //connect to the column pinouts of the keypad

Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled(U8G2_R0);

unsigned long now,lastPressed;
char keyStr[16];

void setup(){
  oled.begin();
  lastPressed = millis();

  oled.clearBuffer();
  oled.setFont(u8g2_font_smart_patrol_nbp_tf);
  oled.drawStr(0,11,"KeyMatrix");

  oled.setFont(u8g2_font_helvB18_tf);
  oled.drawStr(0,35,"None");    
}
  
void loop(){
  now=millis();
  if (now>lastPressed+1000) {
    oled.clearBuffer();
    oled.setFont(u8g2_font_smart_patrol_nbp_tf);
    oled.drawStr(0,11,"KeyMatrix");
    oled.setFont(u8g2_font_helvB18_tf);
    oled.drawStr(0,35,"None");    
    oled.sendBuffer();
  }
  char key = keypad.getKey();  
  if (key) {
    lastPressed=now;
    oled.clearBuffer();
    oled.setFont(u8g2_font_smart_patrol_nbp_tf);
    //oled.drawStr(0,11,"KeyMatrix");
    sprintf(keyStr,"KeyMatrix %d",key);
    oled.drawStr(0,11,keyStr);
    oled.setFont(u8g2_font_helvB18_tf);    
    switch (key) {
      case 1: // Cycle Start = NUM_-
        sprintf(keyStr,"Start",key);
        Keyboard.press(KEYPAD_MINUS);
        Keyboard.release(KEYPAD_MINUS);      
        break;
      case 2: // Feed Hold(Pause) = NUM_+
        sprintf(keyStr,"Pause",key);
        Keyboard.press(KEYPAD_PLUS);
        Keyboard.release(KEYPAD_PLUS);
        break;
      case 3: // Reset = Ctrl + R
        sprintf(keyStr,"Reset",key);
        Keyboard.set_modifier(MODIFIERKEY_CTRL);
        Keyboard.press(KEY_R);
        Keyboard.release(KEY_R);
        Keyboard.set_modifier(0);
        Keyboard.send_now();
        break;
      case 4: // Disp Tool Path(Graph) = F12
        sprintf(keyStr,"Graph",key);
        Keyboard.press(KEY_F12);
        Keyboard.release(KEY_F12);
        break;
      case 5: // Goto Zero = F5
        sprintf(keyStr,"Goto Zero",key);
        Keyboard.press(KEY_F5);
        Keyboard.release(KEY_F5);
        break;
      case 6: // HOME = F7 
        sprintf(keyStr,"Home",key);
        Keyboard.press(KEY_F7);
        Keyboard.release(KEY_F7);
        break;
        
      case 7: // Page = TAB
        sprintf(keyStr,"Page",key);
        Keyboard.press(KEY_TAB);
        Keyboard.release(KEY_TAB);
        break;
      case 8: // CONTSTEPMPG = F10 
        sprintf(keyStr,"Mode",key);
        Keyboard.press(KEY_F10);
        Keyboard.release(KEY_F10);
        break;
      case 9: // Jog Incr(Jog dist) = NUM_5 
        sprintf(keyStr,"Jog Incr",key);
        Keyboard.press(KEYPAD_5);
        Keyboard.release(KEYPAD_5);
        break;
      case 10: // SINGLE = F4 
        sprintf(keyStr,"Single",key);
        Keyboard.press(KEY_F4);
        Keyboard.release(KEY_F4);
        break; 
      case 11: // SINGLE MPG(try cut) = F11
        sprintf(keyStr,"SingleMPG",key);
        Keyboard.press(KEY_F11);
        Keyboard.release(KEY_F11);
        break;

      case 13: // WORKSPACE(G) = G
        sprintf(keyStr,"Workspace",key);
        Keyboard.press(KEY_G);
        Keyboard.release(KEY_G);
        break;
      case 14: // ZERO = F6
        sprintf(keyStr,"Zero",key);
        Keyboard.press(KEY_F6);
        Keyboard.release(KEY_F6);
        break;
      case 15: // FRO = O not F
        sprintf(keyStr,"FRO",key);
        Keyboard.press(KEY_O);
        Keyboard.release(KEY_O);
        break;
      case 16: // FOCUS = F9
        sprintf(keyStr,"Focus",key);
        Keyboard.press(KEY_F9);
        Keyboard.release(KEY_F9);
        break;
      case 17: // USB DISCO = DEL
        sprintf(keyStr,"USB Disco",key);
        Keyboard.press(KEY_DELETE);
        Keyboard.release(KEY_DELETE);
        break;
  
      case 20: // 1/2 = F3
        sprintf(keyStr,"1/2",key);
        Keyboard.press(KEY_F3);
        Keyboard.release(KEY_F3);
        break;
      case 21: // SRO = P not S
        sprintf(keyStr,"SRO",key);
        Keyboard.press(KEY_P);
        Keyboard.release(KEY_P);
        break;
      case 22: // FOCUS +
        sprintf(keyStr,"+",key);
        Keyboard.press(KEY_EQUAL);
        Keyboard.release(KEY_EQUAL);
        break;
      case 23: // FOCUS -
        sprintf(keyStr,"-",key);
        Keyboard.press(KEY_MINUS);
        Keyboard.release(KEY_MINUS);
        break;
  
      default:
        sprintf(keyStr,"N/A %d",key);
        break;
    }

    oled.drawStr(0,35,keyStr);
    oled.sendBuffer();
    
  }  
  
  
}
